@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998-2001 The Free Software Foundation
+ * Copyright (C) 1998-2002 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -33,24 +33,26 @@ G_BEGIN_DECLS
 typedef GNOME_Database_Row             GdaRow;
 typedef GNOME_Database_RowAttributes   GdaRowAttributes;
 typedef GNOME_Database_FieldAttributes GdaFieldAttributes;
-typedef GNOME_Database_Type            GdaType;
 typedef GNOME_Database_Field           GdaField;
-
-#define GDA_TYPE_NULL GNOME_Database_TYPE_NULL
-#define GDA_TYPE_BIGINT GNOME_Database_TYPE_BIGINT
-#define GDA_TYPE_BINARY GNOME_Database_TYPE_BINARY
-#define GDA_TYPE_BOOLEAN GNOME_Database_TYPE_BOOLEAN
-#define GDA_TYPE_DATE GNOME_Database_TYPE_DATE
-#define GDA_TYPE_GEOMETRIC_POINT GNOME_Database_TYPE_GEOMETRIC_POINT
-#define GDA_TYPE_TIME GNOME_Database_TYPE_TIME
-#define GDA_TYPE_TIMESTAMP GNOME_Database_TYPE_TIMESTAMP
-#define GDA_TYPE_DOUBLE GNOME_Database_TYPE_DOUBLE
-#define GDA_TYPE_INTEGER GNOME_Database_TYPE_INTEGER
-#define GDA_TYPE_SINGLE GNOME_Database_TYPE_SINGLE
-#define GDA_TYPE_SMALLINT GNOME_Database_TYPE_SMALLINT
-#define GDA_TYPE_STRING GNOME_Database_TYPE_STRING
-#define GDA_TYPE_TINYINT GNOME_Database_TYPE_TINYINT
-#define GDA_TYPE_UNKNOWN GNOME_Database_TYPE_UNKNOWN
+typedef enum
+{
+	GDA_TYPE_NULL = GNOME_Database_TYPE_NULL,
+	GDA_TYPE_BIGINT = GNOME_Database_TYPE_BIGINT,
+	GDA_TYPE_BINARY = GNOME_Database_TYPE_BINARY,
+	GDA_TYPE_BOOLEAN = GNOME_Database_TYPE_BOOLEAN,
+	GDA_TYPE_DATE = GNOME_Database_TYPE_DATE,
+	GDA_TYPE_GEOMETRIC_POINT = GNOME_Database_TYPE_GEOMETRIC_POINT,
+	GDA_TYPE_TIME = GNOME_Database_TYPE_TIME,
+	GDA_TYPE_TIMESTAMP = GNOME_Database_TYPE_TIMESTAMP,
+	GDA_TYPE_DOUBLE = GNOME_Database_TYPE_DOUBLE,
+	GDA_TYPE_INTEGER = GNOME_Database_TYPE_INTEGER,
+	GDA_TYPE_SINGLE = GNOME_Database_TYPE_SINGLE,
+	GDA_TYPE_SMALLINT = GNOME_Database_TYPE_SMALLINT,
+	GDA_TYPE_TINYINT = GNOME_Database_TYPE_TINYINT,
+	GDA_TYPE_STRING = GNOME_Database_TYPE_STRING,
+	GDA_TYPE_UNKNOWN = GNOME_Database_TYPE_UNKNOWN
+}
+GdaType;
 
 GdaRow             *gda_row_new (gint count);
 void                gda_row_free (GdaRow *row);
@@ -88,11 +90,11 @@ gconstpointer       gda_field_get_binary_value (GdaField *field);
 void                gda_field_set_binary_value (GdaField *field, gconstpointer value, glong size);
 gboolean            gda_field_get_boolean_value (GdaField *field);
 void                gda_field_set_boolean_value (GdaField *field, gboolean value);
-GDate              *gda_field_get_date_value (GdaField *field);
-void                gda_field_set_date_value (GdaField *field, GDate *date);
+const GdaDate      *gda_field_get_date_value (GdaField *field);
+void                gda_field_set_date_value (GdaField *field, GdaDate *date);
 gdouble             gda_field_get_double_value (GdaField *field);
 void                gda_field_set_double_value (GdaField *field, gdouble value);
-GdaGeometricPoint  *gda_field_get_geometric_point_value (GdaField *field);
+const GdaGeometricPoint  *gda_field_get_geometric_point_value (GdaField *field);
 void                gda_field_set_geometric_point_value (GdaField *field, GdaGeometricPoint *value);
 gint                gda_field_get_integer_value (GdaField *field);
 void                gda_field_set_integer_value (GdaField *field, gint value);
@@ -103,14 +105,14 @@ gshort              gda_field_get_smallint_value (GdaField *field);
 void                gda_field_set_smallint_value (GdaField *field, gshort value);
 const gchar        *gda_field_get_string_value (GdaField *field);
 void                gda_field_set_string_value (GdaField *field, const gchar *value);
-GTime               gda_field_get_time_value (GdaField *field);
-void                gda_field_set_time_value (GdaField *field, GTime value);
-time_t              gda_field_get_timestamp_value (GdaField *field);
-void                gda_field_set_timestamp_value (GdaField *field, time_t value);
+const GdaTime      *gda_field_get_time_value (GdaField *field);
+void                gda_field_set_time_value (GdaField *field, GdaTime *value);
+const GdaTimestamp  *gda_field_get_timestamp_value (GdaField *field);
+void                gda_field_set_timestamp_value (GdaField *field, GdaTimestamp *value);
 gchar               gda_field_get_tinyint_value (GdaField *field);
 void                gda_field_set_tinyint_value (GdaField *field, gchar value);
 
-gchar              *gda_field_stringify (GdaField *field);
+gchar              *gda_field_stringify (const GdaField *field);
 
 G_END_DECLS
 
