@@ -1,5 +1,5 @@
 /* GDA - Test suite
- * Copyright (C) 1998-2001 The Free Software Foundation
+ * Copyright (C) 1998-2002 The GNOME Foundation
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -21,6 +21,7 @@
  */
 
 #include "gda-test.h"
+#include "postgres-test.h"
 
 /* Shows the object schemas */
 static void
@@ -44,7 +45,7 @@ show_schema (GdaConnection *cnc, GdaConnectionSchema schema, const gchar *label)
 	for (r = 0; r < row_count; r++) {
 		g_print ("\t");
 		for (c = 0; c < col_count; c++) {
-			GdaValue *value;
+			const GdaValue *value;
 			gchar *str;
 
 			value = gda_data_model_get_value_at (model, c, r);
@@ -66,6 +67,7 @@ open_connection (GdaClient *client,
 {
 	GdaConnection *cnc;
 	gboolean res;
+	GdaDataSourceInfo *dsn_info;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 
@@ -104,6 +106,11 @@ open_connection (GdaClient *client,
 	res = gda_connection_rollback_transaction (cnc, NULL);
 	g_print ("%s\n", res ? "OK" : "Error");
 
+	dsn_info = gda_config_find_data_source (name);
+	/* Postgres own tests */
+	if (!strcmp (dsn_info->provider, POSTGRES_PROVIDER_OAFIID))
+		do_postgres_test (cnc);
+
 	/* close the connection */
 	gda_connection_close (cnc);
 }
